@@ -22,21 +22,12 @@ import logging
 
 from .kalman import skf, fis, miss_data
 from .config import DFMConfig
-from .dfm import DFMResult, calculate_rmse
+from .dfm import DFMResult
 
 # Set up logger
 _logger = logging.getLogger(__name__)
 
-# ============================================================================
-# Constants
-# ============================================================================
-
-# Number of months to append for forecasting horizon
-# Used when extending the time index for forecast generation
 FORECAST_HORIZON_MONTHS: int = 12
-
-# Default fallback date for time index extension
-# Used when the end date cannot be determined from the data
 DEFAULT_FALLBACK_DATE: str = '2017-01-01'
 
 
@@ -55,8 +46,21 @@ def _check_config_consistency(saved_config: Any, current_config: DFMConfig) -> N
     Issues a UserWarning if configs differ. Logs warnings if comparison fails.
     """
     try:
-        if hasattr(saved_config, 'SeriesID') and hasattr(current_config, 'SeriesID'):
-            if saved_config.SeriesID != current_config.SeriesID:
+        # Compare series IDs using new API
+        if hasattr(saved_config, 'get_series_ids'):
+            saved_ids = saved_config.get_series_ids()
+        elif hasattr(saved_config, 'SeriesID'):
+            saved_ids = saved_config.SeriesID
+        else:
+            saved_ids = []
+        
+        if hasattr(current_config, 'get_series_ids'):
+            current_ids = current_config.get_series_ids()
+        elif hasattr(current_config, 'SeriesID'):
+            current_ids = current_config.SeriesID
+        else:
+            current_ids = []
+        if saved_ids != current_ids:
                 import warnings
                 warnings.warn(
                     "Config used in estimation differs from current config. "
@@ -112,12 +116,8 @@ def para_const(X: np.ndarray, P: Union[DFMResult, Dict[str, Any]], lag: int) ->
     
     Plag = [Vs]
     
-    # Calculate lagged covariance matrices
-    # MATLAB loops backwards: for jt = size(Plag{1},3):-1:lag+1
     for jk in range(1, lag + 1):
         Plag_jk = np.zeros_like(Plag[0])
-        # Loop backwards from last time index down to lag+1 (MATLAB: size(Plag{1},3):-1:lag+1)
-        # Note: Python uses 0-based indexing, so range(N-1, lag, -1) gives N-1 down to lag+1
         for jt in range(Plag[0].shape[2] - 1, lag, -1):  # Backward iteration to match MATLAB
             As = Vf[:, :, jt - jk] @ A.T @ pinv(A @ Vf[:, :, jt - jk] @ A.T + Q)
             Plag_jk[:, :, jt] = As @ Plag[jk - 1][:, :, jt]
@@ -163,7 +163,7 @@ def news_dfm(X_old: np.ndarray, X_new: np.ndarray, Res: Union[DFMResult, Dict[st
     Res : Union[DFMResult, Dict[str, Any]]
         DFM estimation results. Can be:
         - DFMResult: Direct result object from dfm() function
-        - Dict: Dictionary containing 'Res' key with DFMResult, and optionally 'Config' or 'Spec'
+        - Dict: Dictionary containing 'Res' key with DFMResult, and optionally 'Config'
         Must have attributes: C, Mx, Wx, X_sm (or equivalent keys if dict)
     t_fcst : int
         Target time index (0-based) for the forecast period.
@@ -171,7 +171,7 @@ def news_dfm(X_old: np.ndarray, X_new: np.ndarray, Res: Union[DFMResult, Dict[st
     v_news : Union[int, np.ndarray, List[int]]
         Target variable index(s) (0-based). Specifies which series to forecast.
         Can be:
-        - int: Single target variable (backward compatible)
+        - int: Single target variable
         - np.ndarray or List[int]: Multiple target variables (MATLAB-compatible)
         All indices must be in range [0, N) where N is number of series.
         
@@ -349,7 +349,7 @@ def news_dfm(X_old: np.ndarray, X_new: np.ndarray, Res: Union[DFMResult, Dict[st
             k = int(k)
             
             C = Res_actual.C
-            R_cov = Res_actual.R.T  # Note: R in Res might need transposition
+            R_cov = Res_actual.R.T
             
             n_news = len(lag)
             
@@ -499,8 +499,8 @@ def update_nowcast(X_old: np.ndarray, X_new: np.ndarray, Time: pd.DatetimeIndex,
     config : DFMConfig
         Model configuration
     Res : DFMResult or dict
-        DFM estimation results. If dict, must contain 'Res' and optionally 'Spec'.
-        If DFMResult and has .Spec attribute, it will be checked for consistency.
+        DFM estimation results. If dict, must contain 'Res' and optionally 'Config'.
+        If DFMResult and has .Config attribute, it will be checked for consistency.
     series : str
         Target series ID
     period : str
@@ -536,15 +536,29 @@ def update_nowcast(X_old: np.ndarray, X_new: np.ndarray, Time: pd.DatetimeIndex,
         raise ValueError(f"update_nowcast: series must be a string, got {type(series)}")
     if not isinstance(period, str):
         raise ValueError(f"update_nowcast: period must be a string, got {type(period)}")
-    if not hasattr(config, 'SeriesID'):
-        raise ValueError(f"update_nowcast: config must have 'SeriesID' attribute")
-    if series not in config.SeriesID:
-        raise ValueError(f"update_nowcast: series '{series}' not found in config.SeriesID")
+    # Get series IDs and find index
+    if hasattr(config, 'get_series_ids'):
+        series_ids = config.get_series_ids()
+    elif hasattr(config, 'SeriesID'):
+        series_ids = config.SeriesID
+    else:
+        series_ids = []
+    
+    if not series_ids:
+        raise ValueError(f"update_nowcast: config must have series definitions")
+    if series not in series_ids:
+        raise ValueError(f"update_nowcast: series '{series}' not found in config")
     
     # Validate period format
     try:
-        i_series = config.SeriesID.index(series)
-        freq = config.Frequency[i_series]
+        i_series = series_ids.index(series)
+        if hasattr(config, 'get_frequencies'):
+            frequencies = config.get_frequencies()
+        elif hasattr(config, 'Frequency'):
+            frequencies = config.Frequency
+        else:
+            frequencies = []
+        freq = frequencies[i_series] if i_series < len(frequencies) else 'm'
         if freq == 'm':
             # Format: YYYYmMM (e.g., "2016m12")
             parts = period.split('m')
@@ -570,22 +584,15 @@ def update_nowcast(X_old: np.ndarray, X_new: np.ndarray, Time: pd.DatetimeIndex,
     
     # Handle dict format (from saved results) - extract Res and check config consistency
     if isinstance(Res, dict):
-        if 'Spec' in Res and 'Res' in Res:
-            # Legacy: Spec key for backward compatibility
-            saved_config = Res['Spec']
-            Res = Res['Res']
-            # Simple consistency check: compare SeriesID
-            _check_config_consistency(saved_config, config)
-        elif 'Config' in Res and 'Res' in Res:
-            # New format with Config key
+        if 'Config' in Res and 'Res' in Res:
             saved_config = Res['Config']
             Res = Res['Res']
             _check_config_consistency(saved_config, config)
         elif 'Res' in Res:
             Res = Res['Res']
     
-    # Also check if DFMResult has a Config attribute (for future compatibility)
-    if hasattr(Res, 'Config') and hasattr(config, 'SeriesID'):
+    # Check config consistency if Res has Config attribute
+    if hasattr(Res, 'Config'):
         _check_config_consistency(Res.Config, config)
     
     # Make sure datasets are same size
@@ -624,13 +631,33 @@ def update_nowcast(X_old: np.ndarray, X_new: np.ndarray, Time: pd.DatetimeIndex,
         Time = pd.concat([Time, pd.Index(new_dates)])
     
     # Find series index
+    if hasattr(config, 'get_series_ids'):
+        series_ids = config.get_series_ids()
+    elif hasattr(config, 'SeriesID'):
+        series_ids = config.SeriesID
+    else:
+        series_ids = []
+    
     try:
-        i_series = config.SeriesID.index(series)
+        i_series = series_ids.index(series)
     except ValueError:
         raise ValueError(f"Series {series} not found in configuration")
     
-    series_name = config.SeriesName[i_series]
-    freq = config.Frequency[i_series]
+    if hasattr(config, 'get_series_names'):
+        series_names = config.get_series_names()
+    elif hasattr(config, 'SeriesName'):
+        series_names = config.SeriesName
+    else:
+        series_names = []
+    
+    if hasattr(config, 'get_frequencies'):
+        frequencies = config.get_frequencies()
+    elif hasattr(config, 'Frequency'):
+        frequencies = config.Frequency
+    else:
+        frequencies = []
+    series_name = series_names[i_series] if i_series < len(series_names) else series
+    freq = frequencies[i_series] if i_series < len(frequencies) else 'm'
     
     # Parse period
     if freq == 'm':
@@ -678,7 +705,14 @@ def update_nowcast(X_old: np.ndarray, X_new: np.ndarray, Time: pd.DatetimeIndex,
     else:
         period_str = pd.Timestamp(Time[t_nowcast]).strftime('%Y-%m')
     
-    print(f'Nowcast for {series_name} ({config.UnitsTransformed[i_series]}), {period_str}')
+    # Get units
+    if hasattr(config, 'series') and i_series < len(config.series):
+        units = config.series[i_series].units
+    elif hasattr(config, 'UnitsTransformed') and i_series < len(config.UnitsTransformed):
+        units = config.UnitsTransformed[i_series]
+    else:
+        units = ""
+    print(f'Nowcast for {series_name} ({units}), {period_str}')
     
     if len(forecast) == 0 or np.all(np.isnan(forecast)):
         print('\n  No forecast was made.\n')
@@ -703,7 +737,13 @@ def update_nowcast(X_old: np.ndarray, X_new: np.ndarray, Time: pd.DatetimeIndex,
         print('\n  Nowcast Detail Table \n')
         for i in range(N):
             if data_released[i] and not np.isnan(forecast[i]):
-                series_id = config.SeriesID[i]
+                if hasattr(config, 'get_series_ids'):
+                    series_ids = config.get_series_ids()
+                elif hasattr(config, 'SeriesID'):
+                    series_ids = config.SeriesID
+                else:
+                    series_ids = []
+                series_id = series_ids[i] if i < len(series_ids) else f"series_{i}"
                 print(f'{series_id:20s}  Forecast: {forecast[i]:8.2f}  Actual: {actual[i]:8.2f}  '
                       f'Weight: {weight[i]:8.4f}  Impact: {impact_releases[i]:8.2f}')
         
